//***************************************************************************
// Group Linslot / Linux - Slotrace Manager
// File iothread.hpp
// Date 28.11.06 - Jrg Wendel
// This code is distributed under the terms and conditions of the
// GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
//***************************************************************************

#ifndef _IO_THREAD_H_
#define _IO_THREAD_H_

//***************************************************************************
// Includes
//***************************************************************************

#include <QThread>
#include <QMutex>

#include <iow.hpp>

  //#define __TEST_MODE

//***************************************************************************
// Slot Service
//***************************************************************************

class SlotService
{
   public:

      // declarations

#     pragma pack(push, 1)

      struct IowValue
      {
         union
         {
            unsigned int dword;
            unsigned char bytes[4];
         };
      };

#     pragma pack(pop)

      struct IoEvent
      {
         unsigned char byte;
         timeval tp;
      };

      // convert

      static int isBit(unsigned char byte, int bit);
      static void toggleBit(unsigned char &byte, int bit);
      static void setBitTo(unsigned char &byte, int bit, int value);

      static const char* toBinStr(unsigned char byte);
      static double kmh(double meter, double usec);
      static long elapsed(const timeval* tpFrom, const timeval* tpNow);
      static const char* secStr(long usec);
};

//***************************************************************************
// I/O Thread
//***************************************************************************

class IoThread : public QThread, public SlotService
{
      Q_OBJECT
      
   public:

      // object
      
      IoThread();
      virtual ~IoThread();

      // interface

      int init(const char* dev);
      int exit();
      int open();
      int close();
      int reopen();
      void stop()                      { running = false; }
      void setTestMode(int aFlag)      { testMode = aFlag; }
      void writeBit(int Bit, int value);
      int readBit(int Bit);

   signals:

      void onIoSignal(const IoEvent &ioEvent);

   protected:
      
      void run();

      // I/O

      int flushStack(Iow::IowHandle io);
      int lookByte(Iow::IowHandle io, int byteNum, unsigned  char* byte);
      int waitByte(Iow::IowHandle io, int byteNum, unsigned  char* byte);
      int writeByte(Iow::IowHandle devHandle, int byteNum, unsigned char byte);

      // data

      int testMode;
      Iow* iow;
      Iow::IowHandle devHandle;
      Iow::IowHandle io;
      QMutex mutex;
      int running;

      unsigned char out[2];
      char device[100+1];
};

//***************************************************************************
#endif // _IO_THREAD_H_
