
//***************************************************************************
// Group Linslot / Linux - Slotrace Manager
// File iow.hpp
// Date 05.11.06 - Jrg Wendel
// This code is distributed under the terms and conditions of the
// GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
//***************************************************************************

#ifndef _IOW_H_
#define _IOW_H_

//***************************************************************************
// Include
//***************************************************************************

#include "iowarrior.h"

//***************************************************************************
// IO Warrior
//***************************************************************************

class Iow
{

   public:

      // declarations

      typedef void* IowHandle;

      enum Misc
      {
         maxDevices = 16
      };

      struct IowDevice
      {
         int dev_num;                      // numeric part of device name for pipe 0
         iowarrior_info info;              // All the information available for this device
         int fdIO;                         // the filedescriptor for the io-pins
         timeval readTimeout;              // timeout for reading from the device
         timeval writeTimeout;             // timeout for writing to the device, not used on linux
         unsigned int readTimeoutVal;      // read timeout from SetTimeout
         unsigned int writeTimeoutVal;     // write timeout from SetWriteTimeout
         unsigned int lastValue;
      };

#     pragma pack(push, 1)

      struct Report
      {
         unsigned char ReportID;
         
         union
         {
            unsigned int Value;
            unsigned char Bytes[4];
         };
      };

#     pragma pack(pop)

      struct Report24
      {
         unsigned char ReportID;
         unsigned char Bytes[2];
      };

      // object

      Iow();

      // interface

      IowHandle OpenDevice(const char* dev);
      void CloseDevice(IowHandle devHandle);

      int Write(IowHandle devHandle, char* buffer, int length);
      int Read(IowHandle devHandle, char* buffer, int length);
      bool ReadImmediate(IowHandle devHandle, unsigned int* value);

      int GetNumDevs()    { return numIoWarriors; }
      IowHandle GetDeviceHandle(unsigned int numDevice);

      bool SetTimeout(IowHandle devHandle, unsigned int timeout);
      bool SetWriteTimeout(IowHandle devHandle, unsigned int timeout);

   protected:

      // functions

      void Clear();
      IowDevice* GetDeviceByHandle(IowHandle iowHandle);

      // data

      IowDevice _IowDevice;
      unsigned int numIoWarriors;
      IowDevice IoWarriors[maxDevices];
};

//***************************************************************************
#endif // _IOW_H_
