//***************************************************************************
// Group Linslot / Linux - Slotrace Manager
// File linslot.hpp
// Date 05.12.06 - Jrg Wendel
// This code is distributed under the terms and conditions of the
// GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
//***************************************************************************

#ifndef _LINSLOT_H_
#define _LINSLOT_H_

//***************************************************************************
// Includes
//***************************************************************************

#include <QMainWindow>
#include <QThread>
#include <QReadWriteLock>
#include <QTimer>
#include <QDir>

#include <ui_linslot.h>

#include <highscore.hpp>
#include <setup.hpp>
#include <common.hpp>
#include <iothread.hpp>
#include <sqlite.hpp>

//***************************************************************************
// Class LinslotWindow
//***************************************************************************

class LinslotWindow : public QMainWindow, private Ui::LinslotWindow, public SlotService
{
      Q_OBJECT

   public:

      // declarations

      enum Misc
      {
         sizeName       = 50,
         bitsPerByte    = 8,

         bounceTime     = 300000,   // Seconds (0.3 sec)
         bounceTimeSlot = 2000000,  // Seconds (2.0 sec)
         slotCount      = 2,
         
         bitPowerSlot1  = 9,
         bitPowerSlot2  = 8
      };

      enum PowerState
      {
         psSlot1 = 1,
         psSlot2 = 2,

         psAll = psSlot1 | psSlot2
      };

      enum LedState
      {
         lsAllOff     = 0,

         lsGreen      = 1,
         lsRed1       = 2,
         lsRed2       = 4,
         lsRed3       = 8,
         lsRed4       = 16,
         lsRed5       = 32,
         lsYellow1    = 64,
         lsYellow2    = 128,

         lsPhase1     = lsRed1,
         lsPhase2     = lsPhase1 | lsRed2,
         lsPhase3     = lsPhase2 | lsRed3,
         lsPhase4     = lsPhase3 | lsRed4,
         lsPhase5     = lsPhase4 | lsRed5,
         lsPhase6     = lsGreen,
         
         lsSlotPower1 = lsYellow1,
         lsSlotPower2 = lsYellow2,
         lsSlotPower  = lsSlotPower1 | lsSlotPower2,

         lsAllOn      = lsPhase5 | lsGreen | lsSlotPower
      };

      struct Slot
      {
         timeval lastSignal;
         int round;
         char driver[sizeName+1];
         int penalty;
      };

      // object

      LinslotWindow();
      ~LinslotWindow();

      void init();
      void exit();

   protected:

      // functions

      void atSlotSignal(int slot, const timeval* tp);
      void atStartCountdown();
      void atStart();
      void atFinish(int slot);
      void atAbort(const char* info);
      void atJumpTheGun(int slot);
      void atStop(const char* info);

      int getImagesFor(int led, const char* &ledOn, const char* &ledOff);
      void setLeds(int mask);
      void switchLeds(int mask, int state);

      void setSlotPower(int slot, int flag);
      int bounceCheck(IoEvent &ioEvent);
      void resetWidgets();

      // db stuff

      int openDb();
      int createDb();
      int saveRace();
      int showDb();
      int getDriverId(const char* driver);
      int getCourseId();

      // data

      int testMode;
      IoThread* thread;
      QTimer* timer;
      QTimer* timerElapsed;
      QTimer* timerPenalty;
      Slot theSlots[slotCount];
      timeval raceStart;
      int raceRunning;
      int countdownStarted;
      int countdown;
      QObject* ledsParent;
      double fastLapTime;
      
      SqliteDb* db;

      char* resourcePath;
      QString configPath;

      // dialogs
      
      SetupDialog* setupDialog;
      HighscoreDialog* highscoreDialog;
      
   private slots:

      void onIoSignal(const IoEvent &ioEvent);
      void on_pushButtonPower_clicked();
      void on_pushButtonSaveToDb_clicked();
      void on_pushButtonHallOfFame_clicked();
      void on_pushButtonStartRace_clicked();
      void on_pushButtonOptions_clicked();
      void on_pushButtonInitUsb_clicked();
      void onTimer();
      void onElapsedTimer();
      void onPenaltyTimer();
      void onOptionsAccepted();
      void on_comboBoxDriver1_currentIndexChanged(QString value);
      void on_comboBoxDriver2_currentIndexChanged(QString value);
};

//***************************************************************************
#endif // _LINSLOT_H_

